@@ -17,26 +17,29 @@ package PkgConfig::UDefs;
 
 package PkgConfig;
 
-our $VERSION = '0.03_0';
+#First two digits are Perl version, second two are pkg-config version
+our $VERSION = '0.07520_01';
 
-require 5.005;
+require 5.006;
 
 use strict;
 use warnings;
+use Config;
 use File::Spec;
-use Getopt::Long;
+use File::Glob 'bsd_glob';
 use Class::Struct; #in core since 5.004
-our $UseDebugging;
-
 use Data::Dumper;
+use File::Basename qw( dirname );
+
+our $UseDebugging;
 
 ################################################################################
 ### Check for Log::Fu                                                        ###
 ################################################################################
 BEGIN {
-    my $ret = eval {
-        require Log::Fu;
-        Log::Fu->import({level => "warn"}); 1;
+    my $ret = eval q{
+        use Log::Fu 0.25 { level => "warn" };
+        1;
     };
     
     if(!$ret) {
@@ -58,11 +61,105 @@ our $VarClassSerial = 0;
 ### Sane Defaults                                                            ###
 ################################################################################
 our @DEFAULT_SEARCH_PATH = qw(
-    /usr/lib/pkgconfig /usr/share/pkgconfig
     /usr/local/lib/pkgconfig /usr/local/share/pkgconfig
+    /usr/lib/pkgconfig /usr/share/pkgconfig
 
 );
-push @DEFAULT_SEARCH_PATH, split(/:/, $ENV{PKG_CONFIG_PATH} || "");
+
+if($^O =~ /^(gnukfreebsd|linux)$/ && -r "/etc/debian_version") {
+    if(-x "/usr/bin/dpkg-architecture") {
+        my $arch = `/usr/bin/dpkg-architecture -qDEB_HOST_MULTIARCH`;
+        chomp $arch;
+        @DEFAULT_SEARCH_PATH = (
+            "/usr/local/lib/$arch/pkgconfig",
+            "/usr/local/lib/pkgconfig",
+            "/usr/local/share/pkgconfig",
+            "/usr/lib/$arch/pkgconfig",
+            "/usr/lib/pkgconfig",
+            "/usr/share/pkgconfig",
+        );
+    } else {
+        # TODO: the debian 6 official pkg-config also includes
+        # /usr/local/lib/pkgconfig/x86_64-linux-gnu
+        # /usr/lib/pkgconfig/x86_64-linux-gnu    
+        # but not sure if they are used
+    }
+
+} elsif($^O eq 'linux' && -r "/etc/redhat-release") {
+
+    if(-d "/usr/lib64/pkgconfig") {
+        @DEFAULT_SEARCH_PATH = qw(
+            /usr/lib64/pkgconfig
+            /usr/share/pkgconfig
+        );
+    } else {
+        @DEFAULT_SEARCH_PATH = qw(
+            /usr/lib/pkgconfig
+            /usr/share/pkgconfig
+        );
+    }
+
+} elsif($^O eq 'freebsd') {
+
+    # TODO: FreeBSD 10's version of pkg-config does not
+    # support PKG_CONFIG_DEBUG_SPEW so I can't verify
+    # the path there, but this is what it is for
+    # FreeBSD 9
+    @DEFAULT_SEARCH_PATH = qw(
+        /usr/local/libdata/pkgconfig
+        /usr/local/lib/pkgconfig
+    );
+
+} elsif($^O eq 'netbsd') {
+
+    @DEFAULT_SEARCH_PATH = qw(
+        /usr/pkg/lib/pkgconfig
+        /usr/pkg/share/pkgconfig
+        /usr/X11R7/lib/pkgconfig
+        /usr/lib/pkgconfig
+    );
+} elsif($^O eq 'openbsd') {
+
+    @DEFAULT_SEARCH_PATH = qw(
+        /usr/lib/pkgconfig
+        /usr/local/lib/pkgconfig
+        /usr/local/share/pkgconfig
+        /usr/X11R6/lib/pkgconfig
+        /usr/X11R6/share/pkgconfig
+    );
+
+} elsif($^O eq 'MSWin32') {
+
+    # Caveats:
+    # 1. This pulls in Config,
+    #    which we don't load on non MSWin32
+    #    but it is in the core.
+    # 2. Slight semantic difference in that we are treating
+    #    Strawberry as the "system" rather than Windows, but
+    #    since pkg-config is rarely used in MSWin32, it is
+    #    better to have something that is useful rather than
+    #    worry about if it is exactly the same as other
+    #    platforms.
+    # 3. It is a little brittle in that Strawberry might 
+    #    one day change its layouts.  If it has and you are
+    #    reading this, please send a pull request or simply
+    #    let me know -plicease
+    require Config;
+    if($Config::Config{myuname} =~ /strawberry-perl/)
+    {
+        my($vol, $dir, $file) = File::Spec->splitpath($^X);
+        my @dirs = File::Spec->splitdir($dir);
+        splice @dirs, -3;
+        my $path = (File::Spec->catdir($vol, @dirs, qw( c lib pkgconfig )));
+        $path =~ s{\\}{/}g;
+        @DEFAULT_SEARCH_PATH = $path;
+    }
+
+}
+
+my @ENV_SEARCH_PATH = split($Config{path_sep}, $ENV{PKG_CONFIG_PATH} || "");
+
+unshift @DEFAULT_SEARCH_PATH, @ENV_SEARCH_PATH;
 
 our @DEFAULT_EXCLUDE_CFLAGS = qw(-I/usr/include -I/usr/local/include);
 # don't include default link/search paths!
@@ -172,9 +269,12 @@ struct(
      # classes used for storing persistent data
      'varclass' => '$',
      'udefclass' => '$',
+     'filevars' => '*%',
+     'uservars' => '*%',
      
      # options for printing variables
      'print_variables' => '$',
+     'print_variable' => '$',
      'print_values' => '$',
      'defined_variables' => '*%',
     ]
@@ -220,7 +320,10 @@ sub assign_var {
                     $self->_get_pc_varname($field), $value);
     
     log_debug("EVAL", $evalstr);
-    eval $evalstr;
+    do {
+        no warnings 'uninitialized';
+        eval $evalstr;
+    };
     if($@) {
         log_err($@);
     }
@@ -288,9 +391,22 @@ sub find {
         @libraries = ($library);
     }
     
-    foreach my $lib (@libraries) {
-        $o->recursion(0);
-        $o->find_pcfile($lib);
+    if($options{file_path}) {
+    
+        if(-r $options{file_path}) {
+            $o->recursion(1);
+            $o->parse_pcfile($options{file_path});
+            $o->recursion(0);
+        } else {
+            $o->errmsg("No such file $options{file_path}\n");
+        }
+    
+    } else {
+    
+        foreach my $lib (@libraries) {
+            $o->recursion(0);
+            $o->find_pcfile($lib);
+        }
     }
     
     return $o;
@@ -402,22 +518,24 @@ sub parse_line {
 
     log_debugf("Field %s, Value %s", $field, $value);
     
+    $field = lc($field);
+    
     #perl variables can't have '.' in them:
     $field =~ s/\./DOT/g;
     
-    
-    $field = lc($field);
-    
     #remove quoutes from field names
     $field =~ s/['"]//g;
     
 
     # pkg-config escapes a '$' with a '$$'. This won't go in perl:
     $value =~ s/[^\\]\$\$/\\\$/g;
+    $value =~ s/([@%&])/\$1/g;
     
     
     # append our pseudo-package for persistence.
     my $varclass = $self->varclass;
+    $value =~ s/(\$\{[^}]+\})/lc($1)/ge;
+    
     $value =~ s/\$\{/\$\{$varclass\::/g;
     
     #quoute the value string, unless quouted already
@@ -442,9 +560,18 @@ sub parse_pcfile {
     my @lines = (<$fh>);
     close($fh);
     
+    my $text = join("", @lines);
+    $text =~ s,\\[\r\n],,g;
+    @lines = split(/[\r\n]/, $text);
+    
     my @eval_strings;
     
-    foreach my $line (@lines) {
+    #Fold lines:
+    
+    my $pcfiledir = dirname $pcfile;
+    $pcfiledir =~ s{\\}{/}g;
+
+    foreach my $line ("pcfiledir=$pcfiledir", @lines) {
         $self->parse_line($line, \@eval_strings);
     }
     
@@ -555,6 +682,22 @@ sub get_ldflags {
     return @ret;
 }
 
+sub get_list {
+    my $self = shift;
+    my @search_paths = @{$self->search_path};
+    my @rv = ();
+    $self->recursion(0);
+    for my $d (@search_paths) {
+        next unless -d $d;
+        for my $pc (bsd_glob("$d/*.pc")) {
+            if ($pc =~ m|/([^\\\/]+)\.pc$|) {
+                $self->parse_pcfile($pc);
+                push @rv, [$1, $self->_pc_var('name') . ' - ' . $self->_pc_var('description')];
+            }
+        }
+    }
+    return @rv;
+}
 
 
 ################################################################################
@@ -658,10 +801,11 @@ if(caller) {
 package PkgConfig::Script;
 use strict;
 use warnings;
-use Getopt::Long;
+use Getopt::Long qw(:config no_ignore_case);
 use Pod::Usage;
 
 my $quiet_errors = 1;
+my @ARGV_PRESERVE = @ARGV;
 
 my @POD_USAGE_SECTIONS = (
     "NAME",
@@ -677,10 +821,16 @@ my @POD_USAGE_OPTIONS = (
 
 GetOptions(
     'libs' => \my $PrintLibs,
+    'libs-only-L' => \my $PrintLibsOnlyL,
+    'libs-only-l' => \my $PrintLibsOnlyl,
+    'list-all' => \my $ListAll,
     'static' => \my $UseStatic,
     'cflags' => \my $PrintCflags,
     'exists' => \my $PrintExists,
-    
+    'atleast-version=s' => \my $AtLeastVersion,
+    'exact-version=s'   => \my $ExactVersion,
+    'max-version=s'     => \my $MaxVersion,
+
     'silence-errors' => \my $SilenceErrors,
     'print-errors' => \my $PrintErrors,
     
@@ -688,7 +838,7 @@ GetOptions(
     
     'print-variables' => \my $PrintVariables,
     'print-values'  => \my $PrintValues,
-    'variable=s',   => \my %OutputVariableValue,
+    'variable=s',   => \my $OutputVariableValue,
     
     'modversion'    => \my $PrintVersion,
     'version',      => \my $PrintAPIversion,
@@ -696,6 +846,7 @@ GetOptions(
     
     'debug'         => \my $Debug,
     'with-path=s',    => \my @ExtraPaths,
+    'env-only',     => \my $EnvOnly,
     'guess-paths',  => \my $GuessPaths,
     
     'h|help|?'      => \my $WantHelp
@@ -724,13 +875,11 @@ if($PrintAPIversion) {
 
 if($PrintRealVersion) {
 
-    printf STDOUT ("pkg-config.pl - cruftless pkg-config\n" .
+    printf STDOUT ("ppkg-config - cruftless pkg-config\n" .
             "Version: %s\n", $PkgConfig::VERSION);
     exit(0);
 }
 
-my @FINDLIBS = @ARGV or die "Must specify at least one library";
-
 if($PrintErrors) {
     $quiet_errors = 0;
 }
@@ -738,25 +887,45 @@ if($SilenceErrors) {
     $quiet_errors = 1;
 }
 
-my $WantFlags = ($PrintCflags || $PrintLibs || $PrintVersion);
+my $WantFlags = ($PrintCflags || $PrintLibs || $PrintLibsOnlyL || $PrintLibsOnlyl || $PrintVersion);
 
 if($WantFlags) {
     $quiet_errors = 0 unless $SilenceErrors;
 }
 
 my %pc_options;
-if($PrintExists) {
+if($PrintExists || $AtLeastVersion || $ExactVersion || $MaxVersion) {
     $pc_options{no_recurse} = 1;
 }
 
 
 $pc_options{static} = $UseStatic;
 $pc_options{search_path} = \@ExtraPaths;
+
+if($EnvOnly) {
+    delete $pc_options{search_path};
+    $pc_options{search_path_override} = [ @ExtraPaths, @ENV_SEARCH_PATH];
+}
+
 $pc_options{print_variables} = $PrintVariables;
 $pc_options{print_values} = $PrintValues;
 $pc_options{VARS} = \%UserVariables;
 
+if($ListAll) {
+    my $o = PkgConfig->find([], %pc_options);
+    my @list = $o->get_list();
+    
+    # can't use List::Util::max as it wasn't core until Perl 5.8
+    my $max_length = 0;
+    foreach my $length (map { length $_->[0] } @list) {
+        $max_length = $length if $length > $max_length;
+    }
 
+    printf "%-${max_length}s %s\n", $_->[0], $_->[1] for @list;
+    exit(0); 
+}
+
+my @FINDLIBS = @ARGV or die "Must specify at least one library";
 my $o = PkgConfig->find(\@FINDLIBS, %pc_options);
 
 if($o->errmsg) {
@@ -764,6 +933,31 @@ if($o->errmsg) {
     exit(1);
 }
 
+if($ExactVersion) {
+    exit(2) unless $o->pkg_version eq $ExactVersion;
+}
+
+sub _is_greater_than_or_equal ($$)
+{
+    my @a = split /\./, shift;
+    my @b = split /\./, shift;
+    while(@a || @b) {
+        my $a = shift(@a) || 0;
+        my $b = shift(@b) || 0;
+        return 1 if $a > $b;
+        return 0 if $a < $b;
+    }
+    return 1;
+}
+
+if($AtLeastVersion) {
+    exit(2) unless _is_greater_than_or_equal($o->pkg_version, $AtLeastVersion);
+}
+
+if($MaxVersion) {
+    exit(2) unless _is_greater_than_or_equal($MaxVersion, $o->pkg_version);
+}
+
 if($o->print_variables) {
     while (my ($k,$v) = each %{$o->defined_variables}) {
         print $k;
@@ -775,6 +969,11 @@ if($o->print_variables) {
     }
 }
 
+if($OutputVariableValue) {
+    my $val = ($o->_pc_var($OutputVariableValue) or "");
+    print $val . "\n";
+}
+
 if(!$WantFlags) {
     exit(0);
 }
@@ -792,6 +991,16 @@ if($PrintLibs) {
     print join(" ", $o->get_ldflags) . " ";
 }
 
+# handle --libs-only-L and --libs-only-l but watch the case when
+# we got 'ppkg-config --libs-only-L --libs-only-l foo' which must behave just like
+# 'ppkg-config --libs-only-l foo'
+
+if($PrintLibsOnlyl or ($PrintLibsOnlyl and $PrintLibsOnlyL)) {
+    print grep /^-l/, $o->get_ldflags;
+} elsif ($PrintLibsOnlyL) {
+    print grep /^-[LR]/, $o->get_ldflags;
+}
+
 print "\n";
 exit(0);
 
@@ -799,27 +1008,22 @@ __END__
 
 =head1 NAME
 
-PkgConfig - Pure-Perl Core-Only replacement for C<pkg-config>
-
-=head1 NOTE
-
-The script is not actually installed yet (i haven't settled on a good name), but
-will decide based on input in a future version.
-
-Additionally, some dependencies are superficially included for debugging, and
-will be sanitized in a future 'release/stable' version.
+PkgConfig - Pure-Perl Core-Only replacement for pkg-config
 
 =head1 SYNOPSIS
 
 =head2 As a replacement for C<pkg-config>
 
-    $ pkg-config.pl --libs --cflags --static gio-2.0
+    $ ppkg-config --libs --cflags --static gio-2.0
 
     #outputs (lines artifically broken up for readability):
     # -I/usr/include/glib-2.0 -I/usr/lib/glib-2.0/include
     # -pthread -lgio-2.0 -lz -lresolv -lgobject-2.0
     # -lgmodule-2.0 -ldl -lgthread-2.0 -pthread -lrt -lglib-2.0
 
+C<pkg-config.pl> can be used as an alias for C<ppkg-config> on platforms that
+support it.  It can also be installed as C<pkg-config> though this is not
+recomended if your system has a nativ C<pkg-config>.
 
 Compare to:
     $ pkg-config --libs --cflags --static gio-2.0
@@ -874,10 +1078,26 @@ The output should normally be suitable for passing to your favorite compiler.
 (Also) print linker flags. Dependencies are traverse in order. Top-level dependencies
 will appear earlier in the command line than bottom-level dependencies.
 
+=head4 --libs-only-L
+
+Prints -L/-R part of "--libs". It defines library search path but without libraries to link with.
+
+=head4 --libs-only-l
+
+Prints the -l part of "--libs".
+
+=head4 --list-all
+
+List all know packages.
+
 =head4 --cflags
 
 (Also) print compiler and C preprocessor flags.
 
+=head4 --modversion
+
+Print the version of a given package.
+
 =head4 --static
 
 Use extra dependencies and libraries if linking against a static version of the
@@ -894,6 +1114,11 @@ Prepend C<PATH> to the list of search paths containing C<.pc> files.
 This option can be specified multiple times with different paths, and they will
 all be added.
 
+=head4 --env-only
+
+Using this option, B<only> paths specified in C<PKG_CONFIG_PATH> are recognized
+and any hard-coded defaults are ignored.
+
 =head4 --guess-paths
 
 Invoke C<gcc> and C<ld> to determine default linker and include paths. Default
@@ -904,6 +1129,10 @@ paths will be excluded from explicit -L and -I flags.
 Define a variable, overriding any such variable definition in the .pc file, and
 allowing your value to interpolate with subsequent uses.
 
+=head4 --variable=VARIABLE
+
+This returns the value of a variable defined in a package's .pc file.
+
 =head4 --print-variables
 
 Print all defined variables found in the .pc files.
@@ -961,7 +1190,13 @@ recognized:
 Prepend search paths in addition to the paths specified in C<$ENV{PKG_CONFIG_PATH}>
 The value is an array reference.
 
-the C<_override> variant ignores defaults (like c<PKG_CONFIG_PATH).
+the C<_override> variant ignores defaults (like C<PKG_CONFIG_PATH>).
+
+=item C<file_path>
+
+Specifies the full path of the of the .pc file that you wish to load.  It does
+not need to be in the search path (although any dependencies will need to be).
+Useful if you know the full path of the exact .pc file that you want.
 
 =item C<exclude_cflags>
 
@@ -971,7 +1206,6 @@ the C<_override> variant ignores defaults (like c<PKG_CONFIG_PATH).
 
 =item C<exclude_ldflags_override>
 
-
 Some C<.pc> files specify default compiler and linker search paths, e.g.
 C<-I/usr/include -L/usr/lib>. Specifying them on the command line can be
 problematic as it drastically changes the search order.
@@ -1026,17 +1260,50 @@ paths with those discovered by invoking L<ld(1)> and L<cpp(1)>.
 
 Currently this only works with GCC-supplied C<ld> and GNU C<ld>.
 
+=head2 INSTALL
+
+The C<Makefile.PL> that comes with C<PkgConfig> can take one or more C<--script>
+options to change of the name of the script or scripts that are installed.
+
+=over 4
+
+=item --script ppkg-config
+
+This is the default and works on all platforms
+
+=item --script pkg-config.pl
+
+This is installed by default on all platforms except for Windows, where the .pl
+may confuse the shell and cause the script to be opened in a text editor.
+
+=item --script pkg-config
+
+This is the default name of the real C<pkg-config> and so you have to specifically
+enable it if you want it.
+
+=item --script none
+
+Don't install any scripts.
+
+=back
+
+Example, install all script names:
+
+ % perl Makefile.PL --script ppkg-config --script pkg-config.pl --script pkg-config
+
+Example, don't install any scripts:
+
+ % perl Makefile.PL --script none
+
 =head2 BUGS
 
 The order of the flags is not exactly matching to that of C<pkg-config>. From my
 own observation, it seems this module does a better job, but I might be wrong.
 
-Version checking is not yet implemented.
-
-There is currently a dependency on a debugging module, just to preserve my sanity.
-This will be removed in a future release.
-
-Module tests are missing.
+Unlike C<pkg-config>, the scripts C<--exists> function will return nonzero if
+a package B<or> any of its dependencies are missing. This differs from the
+behavior of C<pkg-config> which will just check for the definition of the
+package itself (without dependencies).
 
 =head1 SEE ALSO
 
@@ -1044,9 +1311,34 @@ L<ExtUtils::PkgConfig>, a wrapper around the C<pkg-config> binary
 
 L<pkg-config|http://www.freedesktop.org/wiki/Software/pkg-config>
 
-=head1 AUTHOR & COPYRIGHT
+L<http://www.openbsd.org/cgi-bin/cvsweb/src/usr.bin/pkg-config/> another 
+perl implementation of pkg-config
+
+=head1 AUTHOR
+
+=over 4
+
+=item Original Author: M. Nunberg
+
+=item Current maintainer: Graham Ollis E<lt>plicease@cpan.orgE<gt>
+
+=back
+
+Other contributors include:
+
+=over 4
+
+=item kmx
+
+=item Sanel Zukan
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2012 M. Nunberg
 
-You may use and distribute this software under the same terms and conditions as
-Perl itself.
+This is free software; you can redistribute it and/or modify it under the same
+terms as the Perl 5 programming language system itself.
+
+=cut
